/* -*- mode: c++; c-basic-offset:4 -*-
    initialization.h

    This file is part of KleopatraClient, the Kleopatra interface library
    SPDX-FileCopyrightText: 2008 Klarälvdalens Datakonsult AB

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include "kleopatraclientcore_export.h"

namespace KleopatraClientCopy
{

class KLEOPATRACLIENTCORE_EXPORT Initialization
{
public:
    Initialization();
    ~Initialization();
};

}
