/*  utils/winapi-helpers.h

    This file is part of Kleopatra, the KDE keymanager
    SPDX-FileCopyrightText: 2024 g10 Code GmbH
    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>

    SPDX-License-Identifier: GPL-2.0-or-later
*/
#pragma once

class QColor;

/**
 * Wrapper for the Windows GetSysColor function.
 */
QColor win_getSysColor(int index);
