/* -*- mode: c++; c-basic-offset:4 -*-
    utils/certificatepair.h

    This file is part of Kleopatra, the KDE keymanager
    SPDX-FileCopyrightText: 2022 g10 Code GmbH
    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once

#include <gpgme++/key.h>

class QDate;

namespace Kleo
{

struct CertificatePair {
    GpgME::Key openpgp;
    GpgME::Key cms;
};

}
